# -*- coding: utf-8 -*-

"""General constants used to spawn a PTY."""


class Backend:
    """Available PTY backends."""
    ConPTY = 0
    WinPTY = 1


class Encoding:
    """Available byte encodings to communicate with a PTY."""
    UTF8 = 'utf-8'
    UTF16 = 'utf-16'


class MouseMode:
    """Mouse capture settings for the winpty backend."""

    # QuickEdit mode is initially disabled, and the agent does not send mouse
    # mode sequences to the terminal.  If it receives mouse input, though, it
    # still writes MOUSE_EVENT_RECORD values into CONIN.
    WINPTY_MOUSE_MODE_NONE = 0

    # QuickEdit mode is initially enabled.  As CONIN enters or leaves mouse
    # input mode (i.e. where ENABLE_MOUSE_INPUT is on and
    # ENABLE_QUICK_EDIT_MODE is off), the agent enables or disables mouse
    # input on the terminal.
    WINPTY_MOUSE_MODE_AUTO = 1

    # QuickEdit mode is initially disabled, and the agent enables the
    # terminal's mouse input mode.  It does not disable terminal
    # mouse mode (until exit).
    WINPTY_MOUSE_MODE_FORCE = 2


class AgentConfig:
    """General configuration settings for the winpty backend."""

    # Create a new screen buffer (connected to the "conerr" terminal pipe) and
    # pass it to child processes as the STDERR handle.  This flag also prevents
    # the agent from reopening CONOUT$ when it polls -- regardless of whether
    # the active screen buffer changes, winpty continues to monitor the
    # original primary screen buffer.
    WINPTY_FLAG_CONERR = 0x1

    # Don't output escape sequences.
    WINPTY_FLAG_PLAIN_OUTPUT = 0x2

    # Do output color escape sequences.  These escapes are output by default,
    # but are suppressed with WINPTY_FLAG_PLAIN_OUTPUT.
    # Use this flag to reenable them.
    WINPTY_FLAG_COLOR_ESCAPES = 0x4



# -*- coding: utf-8 -*-
"""winpty wrapper tests."""

# Standard library imports
import os
import time

# Third party imports
from winpty import PTY, WinptyError
from winpty.enums import Backend
from winpty.ptyprocess import which
import pytest


CMD = bytes(which('cmd').lower(), 'utf-8')


def pty_factory(backend):
    @pytest.fixture(scope='function')
    def pty_fixture():
        pty = PTY(80, 20, backend=backend)
        loc = bytes(os.getcwd(), 'utf8')
        assert pty.spawn(CMD)
        time.sleep(0.3)
        return pty
    return pty_fixture


conpty_provider = pty_factory(Backend.ConPTY)
winpty_provider = pty_factory(Backend.WinPTY)


@pytest.fixture(scope='function', params=[
    pytest.lazy_fixture('conpty_provider'),
    pytest.lazy_fixture('winpty_provider')])
def pty_fixture(request):
    pty = request.param
    return pty


def test_read(pty_fixture, capsys):
    pty = pty_fixture
    loc = os.getcwd()
    readline = ''

    with capsys.disabled():
        start_time = time.time()
        while loc not in readline:
            if time.time() - start_time > 5:
                break
            readline += pty.read().decode('utf-8')
    assert loc in readline


def test_write(pty_fixture):
    pty = pty_fixture
    line = pty.read()

    str_text = 'Eggs, ham and spam ünicode'
    text = bytes(str_text, 'utf-8')
    num_bytes = pty.write(text)

    line = ''
    start_time = time.time()
    while str_text not in line:
        if time.time() - start_time > 5:
            break
        line += pty.read().decode('utf-8')

    assert str_text in line


def test_isalive(pty_fixture):
    pty = pty_fixture
    pty.write(b'exit\r\n')

    text = 'exit'
    line = ''
    while text not in line:
        line += pty.read().decode('utf-8')

    while pty.isalive():
        pty.read()
        continue

    assert not pty.isalive()


def test_agent_spawn_fail(pty_fixture):
    pty = pty_fixture
    try:
        pty.spawn(CMD)
        assert False
    except WinptyError:
        pass


@pytest.mark.parametrize(
    'backend_name,backend',
    [("ConPTY", Backend.ConPTY), ('WinPTY', Backend.WinPTY)])
def test_pty_create_size_fail(backend_name, backend):
    try:
        PTY(80, -25, backend=backend)
        assert False
    except WinptyError:
        pass


def test_agent_resize_fail(pty_fixture):
    pty = pty_fixture
    try:
        pty.set_size(-80, 70)
        assert False
    except WinptyError:
        pass


def test_agent_resize(pty_fixture):
    pty = pty_fixture
    pty.set_size(80, 70)

